/*
 * Copyright (c) 2018-2028, AoKunSang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.csetl.engine;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.search.Hit;
import co.elastic.clients.json.jackson.JacksonJsonpMapper;
import co.elastic.clients.transport.ElasticsearchTransport;
import co.elastic.clients.transport.endpoints.BooleanResponse;
import co.elastic.clients.transport.rest_client.RestClientTransport;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponseInterceptor;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.entity.ContentType;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.message.BasicHeader;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.springblade.csetl.dto.BaseQueryParam;
import org.springblade.csetl.dto.EsQueryParam;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

/**
 * elasticsearch执行器
 *
 * @author wookvn
 * @datetime 2025-02-05 15:16:00
 */
@Slf4j
@Component
public class ElasticsearchExecutor implements SqlExecutor<String> {

    @Override
    public Boolean check(BaseQueryParam queryParam) {
        EsQueryParam esQueryParam = (EsQueryParam) queryParam;
        return connectQuery(esQueryParam, elasticsearchClient -> {
            try {
                BooleanResponse pingReponse = elasticsearchClient.ping();
                return pingReponse.value();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return false;
            }
        });
    }

    @Override
    public String query(BaseQueryParam queryParam) {
        EsQueryParam esQueryParam = (EsQueryParam) queryParam;
        return connectQuery(esQueryParam, elasticsearchClient -> {
            SearchRequest searchRequest = SearchRequest.of(s -> s
                    .index(esQueryParam.getIndexName())
                    .withJson(StrUtil.getReader(esQueryParam.getQuerySql()))
            );
            try {
                SearchResponse<Object> searchResponse = elasticsearchClient.search(searchRequest, Object.class);
                List<Object> resultList = new ArrayList<>();
                for (Hit<Object> hit : searchResponse.hits().hits()) {
                    resultList.add(hit.source());
                }
                return JSONUtil.toJsonStr(resultList);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
            return null;
        });
    }

    private <T> T connectQuery(EsQueryParam esQueryParam, Function<ElasticsearchClient, T> consumer){
        // 创建用户名密码对象
        BasicCredentialsProvider basicCredentialsProvider = new BasicCredentialsProvider();
        basicCredentialsProvider.setCredentials(AuthScope.ANY, new UsernamePasswordCredentials(esQueryParam.getUsername(), esQueryParam.getPassword()));

        // 设置 连接地址和端口
        RestClientBuilder restClientBuilder = RestClient.builder(new HttpHost(esQueryParam.getHost(), esQueryParam.getPort()));
        restClientBuilder.setHttpClientConfigCallback(httpClientBuilder ->
                httpClientBuilder.setDefaultCredentialsProvider(basicCredentialsProvider)
                        .setDefaultHeaders(
                                ListUtil.of(
                                        new BasicHeader(
                                                HttpHeaders.CONTENT_TYPE, ContentType.APPLICATION_JSON.toString())))
                        .addInterceptorLast(
                                (HttpResponseInterceptor)
                                        (response, context) ->
                                                response.addHeader("X-Elastic-Product", "Elasticsearch")));
        // 创建 Elasticsearch 客户端
        try(RestClient restClient = restClientBuilder.build();
            ElasticsearchTransport elasticsearchTransport = new RestClientTransport(restClient, new JacksonJsonpMapper());) {
            ElasticsearchClient elasticsearchClient = new ElasticsearchClient(elasticsearchTransport);
            return consumer.apply(elasticsearchClient);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return null;
    }

//    public static void main(String[] args) {
//        ElasticsearchExecutor ss = new ElasticsearchExecutor();
//        EsQueryParam esQueryParam = new EsQueryParam();
//        esQueryParam.setHost("localhost");
//        esQueryParam.setPort(9200);
//        esQueryParam.setUsername("elastic");
//        esQueryParam.setPassword("Xy7t7yS4dIb6s28IBWqm");
//        esQueryParam.setIndexName("library");
//        esQueryParam.setQuerySql("{\"query\":{\"match_all\":{}}}");
//        System.out.println(ss.query(esQueryParam));
//    }
}
