/*
 * Copyright (c) 2018-2028, AoKunSang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.csetl.engine;

import cn.hutool.core.util.StrUtil;
import com.influxdb.client.InfluxDBClient;
import com.influxdb.client.InfluxDBClientFactory;
import com.influxdb.query.FluxRecord;
import com.influxdb.query.FluxTable;
import lombok.extern.slf4j.Slf4j;
import org.influxdb.InfluxDB;
import org.influxdb.InfluxDBFactory;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.springblade.csetl.dto.BaseQueryParam;
import org.springblade.csetl.dto.InfluxdbQueryParam;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * influxdb执行器
 *
 * @author wookvn
 * @datetime 2025-02-05 15:16:33
 */
@Slf4j
@Component
public class InfluxdbExecutor implements SqlExecutor<List<Map<String, Object>>> {

    @Override
    public Boolean check(BaseQueryParam queryParam) {
        InfluxdbQueryParam influxdbQueryParam = (InfluxdbQueryParam) queryParam;
        if(influxdbQueryParam.isFluxV2()) {
            //如果有token，则不使用userpass
            try(InfluxDBClient client =
                        StrUtil.isNotBlank(influxdbQueryParam.getToken())
                                ? InfluxDBClientFactory.create(InfluxdbQueryParam.urlOf(influxdbQueryParam), influxdbQueryParam.getToken().toCharArray())
                                : InfluxDBClientFactory.create(InfluxdbQueryParam.urlOf(influxdbQueryParam), influxdbQueryParam.getUsername(), influxdbQueryParam.getPassword().toCharArray());){
                return client.ping();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return false;
            }
        } else {
            try(InfluxDB influxdb = InfluxDBFactory.connect(InfluxdbQueryParam.urlOf(influxdbQueryParam), influxdbQueryParam.getUsername(), influxdbQueryParam.getPassword());){
                return influxdb.ping().isGood();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return false;
            }
        }
    }

    @Override
    public List<Map<String, Object>> query(BaseQueryParam queryParam) {
        InfluxdbQueryParam influxdbQueryParam = (InfluxdbQueryParam) queryParam;
        return influxdbQueryParam.isFluxV2() ? fluxQuery(influxdbQueryParam) : influxQLQuery(influxdbQueryParam);
    }

    /**
     * <p>influxdb1.x版本查询方式</p>
     * @author wookvn
     * @datetime 2025-02-05 13:33:33
     * @param influxdbQueryParam: （参数说明）
     **/
    List<Map<String, Object>> influxQLQuery(InfluxdbQueryParam influxdbQueryParam) {
        try(InfluxDB influxDB = InfluxDBFactory.connect(InfluxdbQueryParam.urlOf(influxdbQueryParam), influxdbQueryParam.getUsername(), influxdbQueryParam.getPassword());){
            QueryResult result = influxDB.query(new Query(influxdbQueryParam.getQuerySql(), influxdbQueryParam.getDatabase()));
            List<Map<String, Object>> resultList = new ArrayList<>();
            for (QueryResult.Result res : result.getResults()) {
                for (QueryResult.Series series : res.getSeries()) {
//                    series.getTags();  series.getName();
                    Map<String, Object> dataMap = new HashMap<>();
                    //字段名列表
                    List<String> columns = series.getColumns();
                    for (List<Object> row : series.getValues()) {
                        // 遍历字段（假设查询如 SELECT field1, field2 FROM ...）
                        for (int i = 0; i < row.size(); i++) {
                            String fieldName = columns.get(i);
                            Object value = row.get(i);
                            dataMap.put(fieldName, value);
                        }
                        resultList.add(dataMap);
                    }
                }
            }
            return resultList;
        }catch (Exception e){
            log.error(e.getMessage(), e);
        }
        return null;
    }

    /**
     * <p>flux2.x版本查询方式</p>
     * @author wookvn
     * @datetime 2025-02-05 13:35:52
     * @param influxdbQueryParam: （参数说明）
     **/
    List<Map<String, Object>> fluxQuery(InfluxdbQueryParam influxdbQueryParam) {
        try(InfluxDBClient client =
                    StrUtil.isNotBlank(influxdbQueryParam.getToken())
                            ? InfluxDBClientFactory.create(InfluxdbQueryParam.urlOf(influxdbQueryParam), influxdbQueryParam.getToken().toCharArray())
                            : InfluxDBClientFactory.create(InfluxdbQueryParam.urlOf(influxdbQueryParam), influxdbQueryParam.getUsername(), influxdbQueryParam.getPassword().toCharArray());){
            List<FluxTable> tables = client.getQueryApi().query(influxdbQueryParam.getQuerySql());
            List<Map<String, Object>> resultList = new ArrayList<>();
            for (FluxTable table : tables) {
                Map<String, Object> dataMap = new HashMap<>();
                for (FluxRecord record : table.getRecords()) {
                    dataMap.put(record.getField(), record.getValue());
                }
                resultList.add(dataMap);
            }
            return resultList;
        }catch (Exception e){
            log.error(e.getMessage(), e);
        }
        return null;
    }
}
